@@ -130,14 +130,23 @@ func Main() {
 	deleteBcp := deleteBcpOpts{}
 	deleteBcpCmd.Arg("name", "Backup name").StringVar(&deleteBcp.name)
 	deleteBcpCmd.Flag("older-than", fmt.Sprintf("Delete backups older than date/time in format %s or %s", datetimeFormat, dateFormat)).StringVar(&deleteBcp.olderThan)
+	deleteBcpCmd.Flag("yes", "Don't ask confirmation").Short('y').BoolVar(&deleteBcp.force)
 	deleteBcpCmd.Flag("force", "Force. Don't ask confirmation").Short('f').BoolVar(&deleteBcp.force)
 
 	deletePitrCmd := pbmCmd.Command("delete-pitr", "Delete PITR chunks")
 	deletePitr := deletePitrOpts{}
 	deletePitrCmd.Flag("older-than", fmt.Sprintf("Delete backups older than date/time in format %s or %s", datetimeFormat, dateFormat)).StringVar(&deletePitr.olderThan)
 	deletePitrCmd.Flag("all", "Delete all chunks").Short('a').BoolVar(&deletePitr.all)
+	deletePitrCmd.Flag("yes", "Don't ask confirmation").Short('y').BoolVar(&deletePitr.force)
 	deletePitrCmd.Flag("force", "Force. Don't ask confirmation").Short('f').BoolVar(&deletePitr.force)
 
+	cleanupCmd := pbmCmd.Command("cleanup", "Delete Backups and PITR chunks")
+	cleanupOpts := cleanupOptions{}
+	cleanupCmd.Flag("older-than", fmt.Sprintf("Delete older than date/time in format %s or %s", datetimeFormat, dateFormat)).StringVar(&cleanupOpts.olderThan)
+	cleanupCmd.Flag("yes", "Don't ask confirmation").Short('y').BoolVar(&cleanupOpts.yes)
+	cleanupCmd.Flag("wait", "Wait for deletion done").Short('w').BoolVar(&cleanupOpts.wait)
+	cleanupCmd.Flag("dry-run", "Report but do not delete").BoolVar(&cleanupOpts.dryRun)
+
 	logsCmd := pbmCmd.Command("logs", "PBM logs")
 	logs := logsOpts{}
 	logsCmd.Flag("follow", "Follow output").Short('f').Default("false").BoolVar(&logs.follow)
@@ -221,6 +230,8 @@ func Main() {
 		out, err = deleteBackup(pbmClient, &deleteBcp, pbmOutF)
 	case deletePitrCmd.FullCommand():
 		out, err = deletePITR(pbmClient, &deletePitr, pbmOutF)
+	case cleanupCmd.FullCommand():
+		out, err = retentionCleanup(pbmClient, &cleanupOpts)
 	case logsCmd.FullCommand():
 		out, err = runLogs(pbmClient, &logs)
 	case statusCmd.FullCommand():
@@ -388,6 +399,7 @@ type snapshotStat struct {
 	RestoreTS  int64          `json:"restoreTo"`
 	PBMVersion string         `json:"pbmVersion"`
 	Type       pbm.BackupType `json:"type"`
+	SrcBackup  string         `json:"src"`
 }
 
 type pitrRange struct {
@@ -443,6 +455,8 @@ func cancelBcp(cn *pbm.PBM) (fmt.Stringer, error) {
 	return outMsg{"Backup cancellation has started"}, nil
 }
 
+var errInvalidFormat = errors.New("invalid format")
+
 func parseDateT(v string) (time.Time, error) {
 	switch len(v) {
 	case len(datetimeFormat):
@@ -451,7 +465,7 @@ func parseDateT(v string) (time.Time, error) {
 		return time.Parse(dateFormat, v)
 	}
 
-	return time.Time{}, errors.New("invalid format")
+	return time.Time{}, errInvalidFormat
 }
 
 func findLock(cn *pbm.PBM, fn func(*pbm.LockHeader) ([]pbm.LockData, error)) (*pbm.LockData, error) {
