@@ -159,6 +159,9 @@ func (bl backupListOut) String() string {
 		if len(b.Namespaces) != 0 {
 			kind += ", selective"
 		}
+		if b.Type == pbm.IncrementalBackup && b.SrcBackup == "" {
+			kind += ", base"
+		}
 
 		s += fmt.Sprintf("  %s <%s> [restore_to_time: %s]\n", b.Name, kind, fmtTS(int64(b.RestoreTS)))
 	}
@@ -222,9 +225,18 @@ func getSnapshotList(cn *pbm.PBM, size int, rsMap map[string]string) (s []snapsh
 		return nil, errors.Wrap(err, "define cluster state")
 	}
 
+	ver, err := pbm.GetMongoVersion(cn.Context(), cn.Conn)
+	if err != nil {
+		return nil, errors.WithMessage(err, "get mongo version")
+	}
+	fcv, err := cn.GetFeatureCompatibilityVersion()
+	if err != nil {
+		return nil, errors.WithMessage(err, "get featureCompatibilityVersion")
+	}
+
 	// pbm.PBM is always connected either to config server or to the sole (hence main) RS
 	// which the `confsrv` param in `bcpMatchCluster` is all about
-	bcpsMatchCluster(bcps, shards, inf.SetName, rsMap)
+	bcpsMatchCluster(bcps, ver.VersionString, fcv, shards, inf.SetName, rsMap)
 
 	for i := len(bcps) - 1; i >= 0; i-- {
 		b := bcps[i]
@@ -240,6 +252,7 @@ func getSnapshotList(cn *pbm.PBM, size int, rsMap map[string]string) (s []snapsh
 			RestoreTS:  int64(b.LastWriteTS.T),
 			PBMVersion: b.PBMVersion,
 			Type:       b.Type,
+			SrcBackup:  b.SrcBackup,
 		})
 	}
 
@@ -267,7 +280,7 @@ func getPitrList(cn *pbm.PBM, size int, full, unbacked bool, rsMap map[string]st
 	rsRanges = make(map[string][]pitrRange)
 	var rstlines [][]pbm.Timeline
 	for _, s := range shards {
-		tlns, err := cn.PITRGetValidTimelines(mapRevRS(s.RS), now, nil)
+		tlns, err := cn.PITRGetValidTimelines(mapRevRS(s.RS), now)
 		if err != nil {
 			return nil, nil, errors.Wrapf(err, "get PITR timelines for %s replset", s.RS)
 		}
@@ -327,7 +340,16 @@ func getBaseSnapshotLastWrite(cn *pbm.PBM, sh map[string]bool, rsMap map[string]
 		return nil, nil
 	}
 
-	bcpMatchCluster(bcp, sh, pbm.MakeRSMapFunc(rsMap), pbm.MakeReverseRSMapFunc(rsMap))
+	ver, err := pbm.GetMongoVersion(cn.Context(), cn.Conn)
+	if err != nil {
+		return nil, errors.WithMessage(err, "get mongo version")
+	}
+	fcv, err := cn.GetFeatureCompatibilityVersion()
+	if err != nil {
+		return nil, errors.WithMessage(err, "get featureCompatibilityVersion")
+	}
+
+	bcpMatchCluster(bcp, ver.VersionString, fcv, sh, pbm.MakeRSMapFunc(rsMap), pbm.MakeReverseRSMapFunc(rsMap))
 
 	if bcp.Status != pbm.StatusDone {
 		return nil, nil
