@@ -49,6 +49,14 @@ func (r restoreRet) String() string {
 	case r.err != "":
 		return "\n Error: " + r.err
 	case r.Snapshot != "":
+		if r.physical {
+			return fmt.Sprintf(`
+Restore of the snapshot from '%s' has started.
+Check restore status with: pbm describe-restore %s -c </path/to/pbm.conf.yaml>
+No other pbm command is available while the restore is running!
+`,
+				r.Snapshot, r.Name)
+		}
 		return fmt.Sprintf("Restore of the snapshot from '%s' has started", r.Snapshot)
 	case r.PITR != "":
 		return fmt.Sprintf("Restore to the point in time '%s' has started", r.PITR)
@@ -73,6 +81,12 @@ func runRestore(cn *pbm.PBM, o *restoreOpts, outf outFormat) (fmt.Stringer, erro
 		return nil, errors.New("either a backup name or point in time should be set, non both together!")
 	}
 
+	clusterTime, err := cn.ClusterTime()
+	if err != nil {
+		return nil, errors.Wrap(err, "read cluster time")
+	}
+	tdiff := time.Now().Unix() - int64(clusterTime.T)
+
 	switch {
 	case o.bcp != "":
 		m, err := restore(cn, o.bcp, nss, rsMap, outf)
@@ -80,7 +94,11 @@ func runRestore(cn *pbm.PBM, o *restoreOpts, outf outFormat) (fmt.Stringer, erro
 			return nil, err
 		}
 		if !o.wait {
-			return restoreRet{Name: m.Name, Snapshot: o.bcp}, nil
+			return restoreRet{
+				Name:     m.Name,
+				Snapshot: o.bcp,
+				physical: m.Type == pbm.PhysicalBackup || m.Type == pbm.IncrementalBackup,
+			}, nil
 		}
 
 		typ := " logical restore.\nWaiting to finish"
@@ -88,7 +106,7 @@ func runRestore(cn *pbm.PBM, o *restoreOpts, outf outFormat) (fmt.Stringer, erro
 			typ = " physical restore.\nWaiting to finish"
 		}
 		fmt.Printf("Started%s", typ)
-		err = waitRestore(cn, m)
+		err = waitRestore(cn, m, tdiff)
 		if err == nil {
 			return restoreRet{
 				done:     true,
@@ -109,7 +127,7 @@ func runRestore(cn *pbm.PBM, o *restoreOpts, outf outFormat) (fmt.Stringer, erro
 			return restoreRet{PITR: o.pitr, Name: m.Name}, nil
 		}
 		fmt.Print("Started.\nWaiting to finish")
-		err = waitRestore(cn, m)
+		err = waitRestore(cn, m, tdiff)
 		if err != nil {
 			return restoreRet{err: err.Error()}, nil
 		}
@@ -122,9 +140,15 @@ func runRestore(cn *pbm.PBM, o *restoreOpts, outf outFormat) (fmt.Stringer, erro
 	}
 }
 
-func waitRestore(cn *pbm.PBM, m *pbm.RestoreMeta) error {
+// We rely on heartbeats in error detection in case of all nodes failed,
+// comparing heartbeats with the current cluster time for logical restores.
+// But for physical ones, the cluster by this time is down. So we compare with
+// the wall time taking into account a time skew (wallTime - clusterTime) taken
+// when the cluster time was still available.
+func waitRestore(cn *pbm.PBM, m *pbm.RestoreMeta, tskew int64) error {
 	ep, _ := cn.GetEpoch()
-	stg, err := cn.GetStorage(cn.Logger().NewEvent(string(pbm.CmdRestore), m.Backup, m.OPID, ep.TS()))
+	l := cn.Logger().NewEvent(string(pbm.CmdRestore), m.Backup, m.OPID, ep.TS())
+	stg, err := cn.GetStorage(l)
 	if err != nil {
 		return errors.Wrap(err, "get storage")
 	}
@@ -137,10 +161,15 @@ func waitRestore(cn *pbm.PBM, m *pbm.RestoreMeta) error {
 	getMeta := cn.GetRestoreMeta
 	if m.Type == pbm.PhysicalBackup || m.Type == pbm.IncrementalBackup {
 		getMeta = func(name string) (*pbm.RestoreMeta, error) {
-			return pbm.GetPhysRestoreMeta(name, stg)
+			return pbm.GetPhysRestoreMeta(name, stg, l)
 		}
 	}
 
+	var ctime uint32
+	frameSec := pbm.StaleFrameSec
+	if m.Type != pbm.LogicalBackup {
+		frameSec = 60 * 3
+	}
 	for range tk.C {
 		fmt.Print(".")
 		rmeta, err = getMeta(m.Name)
@@ -151,22 +180,26 @@ func waitRestore(cn *pbm.PBM, m *pbm.RestoreMeta) error {
 			return errors.Wrap(err, "get restore metadata")
 		}
 
-		if m.Type == pbm.LogicalBackup {
-			clusterTime, err := cn.ClusterTime()
-			if err != nil {
-				return errors.Wrap(err, "read cluster time")
-			}
-			if rmeta.Hb.T+pbm.StaleFrameSec < clusterTime.T {
-				return errors.Errorf("operation staled, last heartbeat: %v", rmeta.Hb.T)
-			}
-		}
-
 		switch rmeta.Status {
 		case pbm.StatusDone, pbm.StatusPartlyDone:
 			return nil
 		case pbm.StatusError:
 			return errRestoreFailed{fmt.Sprintf("operation failed with: %s", rmeta.Error)}
 		}
+
+		if m.Type == pbm.LogicalBackup {
+			clusterTime, err := cn.ClusterTime()
+			if err != nil {
+				return errors.Wrap(err, "read cluster time")
+			}
+			ctime = clusterTime.T
+		} else {
+			ctime = uint32(time.Now().Unix() + tskew)
+		}
+
+		if rmeta.Hb.T+frameSec < ctime {
+			return errors.Errorf("operation staled, last heartbeat: %v", rmeta.Hb.T)
+		}
 	}
 
 	return nil
@@ -237,7 +270,7 @@ func restore(cn *pbm.PBM, bcpName string, nss []string, rsMapping map[string]str
 		}
 
 		fn = func(name string) (*pbm.RestoreMeta, error) {
-			return pbm.GetPhysRestoreMeta(name, stg)
+			return pbm.GetPhysRestoreMeta(name, stg, cn.Logger().NewEvent(string(pbm.CmdRestore), bcpName, "", ep.TS()))
 		}
 		ctx, cancel = context.WithTimeout(context.Background(), waitPhysRestoreStart)
 	} else {
@@ -426,12 +459,13 @@ func describeRestore(cn *pbm.PBM, o descrRestoreOpts) (fmt.Stringer, error) {
 			return nil, errors.Wrap(err, "unable to  unmarshal config file")
 		}
 
-		stg, err := pbm.Storage(cfg, log.New(nil, "cli", "").NewEvent("", "", "", primitive.Timestamp{}))
+		l := log.New(nil, "cli", "").NewEvent("", "", "", primitive.Timestamp{})
+		stg, err := pbm.Storage(cfg, l)
 		if err != nil {
 			return nil, errors.Wrap(err, "get storage")
 		}
 
-		meta, err = pbm.GetPhysRestoreMeta(o.restore, stg)
+		meta, err = pbm.GetPhysRestoreMeta(o.restore, stg, l)
 		if err != nil && meta == nil {
 			return nil, errors.Wrap(err, "get restore meta")
 		}
