@@ -551,6 +551,9 @@ func (s storageStat) String() string {
 		if len(sn.Namespaces) != 0 {
 			kind += ", selective"
 		}
+		if sn.Type == pbm.IncrementalBackup && sn.SrcBackup == "" {
+			kind += ", base"
+		}
 
 		ret += fmt.Sprintf("    %s %s <%s> %s\n",
 			sn.Name, fmtSize(sn.Size), kind, status)
@@ -606,6 +609,14 @@ func getStorageStat(cn *pbm.PBM, rsMap map[string]string) (fmt.Stringer, error)
 	if err != nil {
 		return s, errors.Wrap(err, "define cluster state")
 	}
+	ver, err := pbm.GetMongoVersion(cn.Context(), cn.Conn)
+	if err != nil {
+		return nil, errors.WithMessage(err, "get mongo version")
+	}
+	fcv, err := cn.GetFeatureCompatibilityVersion()
+	if err != nil {
+		return nil, errors.WithMessage(err, "get featureCompatibilityVersion")
+	}
 
 	shards, err := cn.ClusterMembers()
 	if err != nil {
@@ -614,7 +625,7 @@ func getStorageStat(cn *pbm.PBM, rsMap map[string]string) (fmt.Stringer, error)
 
 	// pbm.PBM is always connected either to config server or to the sole (hence main) RS
 	// which the `confsrv` param in `bcpMatchCluster` is all about
-	bcpsMatchCluster(bcps, shards, inf.SetName, rsMap)
+	bcpsMatchCluster(bcps, ver.VersionString, fcv, shards, inf.SetName, rsMap)
 
 	stg, err := cn.GetStorage(cn.Logger().NewEvent("", "", "", primitive.Timestamp{}))
 	if err != nil {
@@ -634,6 +645,7 @@ func getStorageStat(cn *pbm.PBM, rsMap map[string]string) (fmt.Stringer, error)
 			RestoreTS:  bcp.LastTransitionTS,
 			PBMVersion: bcp.PBMVersion,
 			Type:       bcp.Type,
+			SrcBackup:  bcp.SrcBackup,
 		}
 		if err := bcp.Error(); err != nil {
 			snpsht.Err = err
@@ -683,16 +695,6 @@ func getPITRranges(cn *pbm.PBM, stg storage.Storage, bcps []pbm.BackupMeta, rsMa
 		return nil, errors.Wrap(err, "get cluster members")
 	}
 
-	fl, err := stg.List(pbm.PITRfsPrefix, "")
-	if err != nil {
-		return nil, errors.Wrap(err, "get chunks list")
-	}
-
-	flist := make(map[string]int64)
-	for _, f := range fl {
-		flist[pbm.PITRfsPrefix+"/"+f.Name] = f.Size
-	}
-
 	now, err := cn.ClusterTime()
 	if err != nil {
 		return nil, errors.Wrap(err, "get cluster time")
@@ -702,7 +704,7 @@ func getPITRranges(cn *pbm.PBM, stg storage.Storage, bcps []pbm.BackupMeta, rsMa
 	var size int64
 	var rstlines [][]pbm.Timeline
 	for _, s := range shards {
-		tlns, err := cn.PITRGetValidTimelines(mapRevRS(s.RS), now, flist)
+		tlns, err := cn.PITRGetValidTimelines(mapRevRS(s.RS), now)
 		if err != nil {
 			return nil, errors.Wrapf(err, "get PITR timelines for %s replset: %s", s.RS, err)
 		}
