@@ -2,9 +2,9 @@ package sharded
 
 import (
 	"fmt"
-	"io/ioutil"
 	"log"
 	"net/url"
+	"os"
 	"strings"
 	"time"
 
@@ -26,7 +26,7 @@ func (c *Cluster) BackupCancellation(storage string) {
 		log.Fatalf("Error: cancel backup '%s'.\nOutput: %s\nStderr:%v", bcpName, o, err)
 	}
 
-	time.Sleep(10 * time.Second)
+	time.Sleep(20 * time.Second)
 
 	checkNoBackupFiles(bcpName, storage)
 
@@ -49,7 +49,7 @@ func (c *Cluster) BackupCancellation(storage string) {
 
 func checkNoBackupFiles(backupName, conf string) {
 	log.Println("check no artifacts left for backup", backupName)
-	buf, err := ioutil.ReadFile(conf)
+	buf, err := os.ReadFile(conf)
 	if err != nil {
 		log.Fatalln("Error: unable to read config file:", err)
 	}
