@@ -5,6 +5,7 @@ import (
 	"time"
 
 	"github.com/percona/percona-backup-mongodb/pbm"
+	"github.com/percona/percona-backup-mongodb/pbm/compress"
 )
 
 // LeaderLag checks if cluster deals with leader lag during backup start
@@ -24,10 +25,10 @@ func (c *Cluster) LeaderLag() {
 	log.Println("Starting backup", bcpName)
 	err = c.mongopbm.SendCmd(pbm.Cmd{
 		Cmd: pbm.CmdBackup,
-		Backup: pbm.BackupCmd{
+		Backup: &pbm.BackupCmd{
 			Type:        pbm.LogicalBackup,
 			Name:        bcpName,
-			Compression: pbm.CompressionTypeS2,
+			Compression: compress.CompressionTypeS2,
 		},
 	})
 	if err != nil {
