@@ -5,15 +5,12 @@ import (
 	"context"
 	"encoding/json"
 	"io"
-	"strings"
 	"time"
 
 	mlog "github.com/mongodb/mongo-tools/common/log"
 	"github.com/pkg/errors"
-	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/mongo"
-	"go.mongodb.org/mongo-driver/mongo/options"
 
 	"github.com/percona/percona-backup-mongodb/pbm"
 	"github.com/percona/percona-backup-mongodb/pbm/compress"
@@ -94,9 +91,16 @@ func (b *Backup) Init(bcp *pbm.BackupCmd, opid pbm.OPID, balancer pbm.BalancerMo
 	meta.Store = cfg.Storage
 
 	ver, err := b.node.GetMongoVersion()
-	if err == nil {
-		meta.MongoVersion = ver.VersionString
+	if err != nil {
+		return errors.WithMessage(err, "get mongo version")
 	}
+	meta.MongoVersion = ver.VersionString
+
+	fcv, err := b.node.GetFeatureCompatibilityVersion()
+	if err != nil {
+		return errors.WithMessage(err, "get featureCompatibilityVersion")
+	}
+	meta.FCV = fcv
 
 	return b.cn.SetBackupMeta(meta)
 }
@@ -189,13 +193,6 @@ func (b *Backup) Run(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OPID, l *
 			}
 		}()
 
-		if inf.IsConfigSrv() && len(bcp.Namespaces) != 0 {
-			err := checkNamespaceForBackup(ctx, b.cn.Conn, bcp.Namespaces[0])
-			if err != nil {
-				return errors.WithMessage(err, "namespace check")
-			}
-		}
-
 		if bcpm.BalancerStatus == pbm.BalancerModeOn {
 			err = b.cn.SetBalancerStatus(pbm.BalancerModeOff)
 			if err != nil {
@@ -270,55 +267,6 @@ func (b *Backup) Run(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OPID, l *
 	return errors.Wrap(err, "waiting for done")
 }
 
-func checkNamespaceForBackup(ctx context.Context, m *mongo.Client, ns string) error {
-	db, coll := parseNS(ns)
-
-	res, err := getShardedNamespaces(ctx, m, db, coll)
-	if err != nil {
-		return errors.WithMessage(err, "get sharded namespaces")
-	}
-	if len(res) != 0 {
-		return errors.Errorf("selective backup: sharded collections: %s", strings.Join(res, ", "))
-	}
-
-	return nil
-}
-
-func getShardedNamespaces(ctx context.Context, m *mongo.Client, db, coll string) ([]string, error) {
-	cur, err := m.Database("config").Collection("collections").
-		Find(ctx, bson.D{}, options.Find().SetProjection(bson.D{{"_id", 1}}))
-	if err != nil {
-		return nil, errors.WithMessage(err, "query")
-	}
-	defer cur.Close(context.Background())
-
-	type document struct {
-		ID string `bson:"_id"`
-	}
-
-	rv := []string{}
-	for cur.Next(ctx) {
-		var doc document
-		if err := cur.Decode(&doc); err != nil {
-			return nil, errors.WithMessage(err, "decode")
-		}
-
-		d, c, _ := strings.Cut(doc.ID, ".")
-		if db != "" && db != d {
-			continue
-		}
-
-		c = strings.TrimPrefix(c, "system.buckets.")
-		if coll != "" && coll != c {
-			continue
-		}
-
-		rv = append(rv, d+"."+c)
-	}
-
-	return rv, nil
-}
-
 func waitForBalancerOff(cn *pbm.PBM, t time.Duration, l *plog.Event) pbm.BalancerMode {
 	dn := time.NewTimer(t)
 	defer dn.Stop()
