@@ -17,13 +17,14 @@ import (
 	"github.com/percona/percona-backup-mongodb/pbm/compress"
 	plog "github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/oplog"
+	"github.com/percona/percona-backup-mongodb/pbm/sel"
 	"github.com/percona/percona-backup-mongodb/pbm/snapshot"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
 )
 
 func (b *Backup) doLogical(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OPID, rsMeta *pbm.BackupReplset, inf *pbm.NodeInfo, stg storage.Storage, l *plog.Event) error {
 	var db, coll string
-	if len(bcp.Namespaces) != 0 {
+	if sel.IsSelective(bcp.Namespaces) {
 		// for selective backup, configsvr does not hold any data.
 		// only some collections from config db is required to restore cluster state
 		if inf.IsConfigSrv() {
@@ -43,7 +44,7 @@ func (b *Backup) doLogical(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OPI
 		}
 	}
 
-	oplog := oplog.NewOplogBackup(b.node)
+	oplog := oplog.NewOplogBackup(b.node.Session())
 	oplogTS, err := oplog.LastWrite()
 	if err != nil {
 		return errors.Wrap(err, "define oplog start position")
@@ -79,7 +80,7 @@ func (b *Backup) doLogical(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OPI
 		return errors.Wrap(err, "waiting for running")
 	}
 
-	if len(bcp.Namespaces) == 0 {
+	if !sel.IsSelective(bcp.Namespaces) {
 		// Save users and roles to the tmp collections so the restore would copy that data
 		// to the system collections. Have to do this because of issues with the restore and preserverUUID.
 		// see: https://jira.percona.com/browse/PBM-636 and comments
@@ -121,9 +122,16 @@ func (b *Backup) doLogical(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OPI
 		return errors.WithMessage(err, "get config")
 	}
 
-	nsFilter := archive.DefaultMatchFunc
-	if len(bcp.Namespaces) != 0 && inf.IsConfigSrv() {
+	nsFilter := archive.DefaultNSFilter
+	docFilter := archive.DefaultDocFilter
+	if inf.IsConfigSrv() && sel.IsSelective(bcp.Namespaces) {
+		chunkSelector, err := createBackupChunkSelector(ctx, b.cn.Conn, bcp.Namespaces)
+		if err != nil {
+			return errors.WithMessage(err, "fetch uuids")
+		}
+
 		nsFilter = makeConfigsvrNSFilter()
+		docFilter = makeConfigsvrDocFilter(bcp.Namespaces, chunkSelector)
 	}
 
 	snapshotSize, err := snapshot.UploadDump(dump,
@@ -140,6 +148,7 @@ func (b *Backup) doLogical(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OPI
 			Compression:      bcp.Compression,
 			CompressionLevel: bcp.CompressionLevel,
 			NSFilter:         nsFilter,
+			DocFilter:        docFilter,
 		})
 	if err != nil {
 		return errors.Wrap(err, "mongodump")
@@ -199,13 +208,74 @@ func (b *Backup) doLogical(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OPI
 	return nil
 }
 
-func makeConfigsvrNSFilter() archive.MatchFunc {
-	// list of required namespaces for further selective restore
-	allowed := map[string]bool{
-		"config.databases": true,
+func createBackupChunkSelector(ctx context.Context, m *mongo.Client, nss []string) (sel.ChunkSelector, error) {
+	ver, err := pbm.GetMongoVersion(ctx, m)
+	if err != nil {
+		return nil, errors.WithMessage(err, "get mongo version")
 	}
 
-	return func(ns string) bool { return allowed[ns] }
+	var chunkSelector sel.ChunkSelector
+	if ver.Major() >= 5 {
+		chunkSelector = sel.NewUUIDChunkSelector()
+	} else {
+		chunkSelector = sel.NewNSChunkSelector()
+	}
+
+	cur, err := m.Database("config").Collection("collections").Find(ctx, bson.D{})
+	if err != nil {
+		return nil, errors.WithMessage(err, "query")
+	}
+	defer cur.Close(ctx)
+
+	selected := sel.MakeSelectedPred(nss)
+	for cur.Next(ctx) {
+		ns := cur.Current.Lookup("_id").StringValue()
+		if selected(ns) {
+			chunkSelector.Add(cur.Current)
+		}
+	}
+	if err := cur.Err(); err != nil {
+		return nil, errors.WithMessage(err, "cursor")
+	}
+
+	return chunkSelector, nil
+}
+
+func makeConfigsvrNSFilter() archive.NSFilterFn {
+	// list of required namespaces for further selective restore
+	allowed := map[string]bool{
+		"config.databases":   true,
+		"config.collections": true,
+		"config.chunks":      true,
+	}
+
+	return func(ns string) bool {
+		return allowed[ns]
+	}
+}
+
+func makeConfigsvrDocFilter(nss []string, selector sel.ChunkSelector) archive.DocFilterFn {
+	selectedNS := sel.MakeSelectedPred(nss)
+	allowedDBs := make(map[string]bool)
+	for _, ns := range nss {
+		db, _, _ := strings.Cut(ns, ".")
+		allowedDBs[db] = true
+	}
+
+	return func(ns string, doc bson.Raw) bool {
+		switch ns {
+		case "config.databases":
+			db, ok := doc.Lookup("_id").StringValueOK()
+			return ok && allowedDBs[db]
+		case "config.collections":
+			ns, ok := doc.Lookup("_id").StringValueOK()
+			return ok && selectedNS(ns)
+		case "config.chunks":
+			return selector.Selected(doc)
+		}
+
+		return false
+	}
 }
 
 func getNamespacesSize(ctx context.Context, m *mongo.Client, db, coll string) (map[string]int64, error) {
@@ -283,16 +353,7 @@ func getNamespacesSize(ctx context.Context, m *mongo.Client, db, coll string) (m
 }
 
 func parseNS(ns string) (string, string) {
-	var db, coll string
-
-	switch s := strings.SplitN(ns, ".", 2); len(s) {
-	case 0:
-		return "", ""
-	case 1:
-		db = s[0]
-	case 2:
-		db, coll = s[0], s[1]
-	}
+	db, coll, _ := strings.Cut(ns, ".")
 
 	if db == "*" {
 		db = ""
