@@ -182,6 +182,21 @@ func (b *Backup) doPhysical(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OP
 				}
 			}
 			currOpts = append(currOpts, bson.E{"srcBackupName", pbm.BackupCursorName(src.Name)})
+		} else {
+			// We don't need any previous incremental backup history if
+			// this is a base backup. So we can flush it to free up resources.
+			l.Debug("flush incremental backup history")
+			cr, err := NewBackupCursor(b.node, l, bson.D{
+				{"disableIncrementalBackup", true},
+			}).create(ctx, cursorCreateRetries)
+			if err != nil {
+				l.Warning("flush incremental backup history error: %v", err)
+			} else {
+				err = cr.Close(ctx)
+				if err != nil {
+					l.Warning("close cursor disableIncrementalBackup: %v", err)
+				}
+			}
 		}
 	}
 	cursor := NewBackupCursor(b.node, l, currOpts)
@@ -261,8 +276,17 @@ func (b *Backup) doPhysical(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OP
 		return errors.Wrap(err, "get journal files")
 	}
 
+	data := bcur.Data
+	stgb, err := getStorageBSON(bcur.Meta.DBpath)
+	if err != nil {
+		return errors.Wrap(err, "check storage.bson file")
+	}
+	if stgb != nil {
+		data = append(data, *stgb)
+	}
+
 	l.Info("uploading data")
-	rsMeta.Journal, rsMeta.Files, err = uploadFiles(ctx, bcur.Data, bcp.Name+"/"+rsMeta.Name, bcur.Meta.DBpath,
+	rsMeta.Files, err = uploadFiles(ctx, data, bcp.Name+"/"+rsMeta.Name, bcur.Meta.DBpath,
 		b.typ == pbm.IncrementalBackup, stg, bcp.Compression, bcp.CompressionLevel, l)
 	if err != nil {
 		return err
@@ -270,13 +294,13 @@ func (b *Backup) doPhysical(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OP
 	l.Info("uploading data done")
 
 	l.Info("uploading journals")
-	ju, _, err := uploadFiles(ctx, jrnls, bcp.Name+"/"+rsMeta.Name, bcur.Meta.DBpath,
+	ju, err := uploadFiles(ctx, jrnls, bcp.Name+"/"+rsMeta.Name, bcur.Meta.DBpath,
 		false, stg, bcp.Compression, bcp.CompressionLevel, l)
 	if err != nil {
 		return err
 	}
-	rsMeta.Journal = append(rsMeta.Journal, ju...)
-	l.Info("uploading journals")
+	l.Info("uploading journals done")
+	rsMeta.Files = append(rsMeta.Files, ju...)
 
 	err = b.cn.RSSetPhyFiles(bcp.Name, rsMeta.Name, rsMeta)
 	if err != nil {
@@ -287,9 +311,6 @@ func (b *Backup) doPhysical(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OP
 	for _, f := range rsMeta.Files {
 		size += f.StgSize
 	}
-	for _, j := range rsMeta.Journal {
-		size += j.StgSize
-	}
 
 	err = b.cn.IncBackupSize(ctx, bcp.Name, size)
 	if err != nil {
@@ -299,6 +320,26 @@ func (b *Backup) doPhysical(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OP
 	return nil
 }
 
+const storagebson = "storage.bson"
+
+func getStorageBSON(dbpath string) (*pbm.File, error) {
+	f, err := os.Stat(path.Join(dbpath, storagebson))
+	if errors.Is(err, os.ErrNotExist) {
+		return nil, nil
+	}
+	if err != nil {
+		return nil, err
+	}
+
+	return &pbm.File{
+		Name:  path.Join(dbpath, storagebson),
+		Len:   f.Size(),
+		Size:  f.Size(),
+		Fmode: f.Mode(),
+	}, nil
+
+}
+
 // UUID represents a UUID as saved in MongoDB
 type UUID struct{ uuid.UUID }
 
@@ -328,21 +369,20 @@ func (id *UUID) IsZero() bool {
 	return bytes.Equal(id.UUID[:], uuid.Nil[:])
 }
 
-const journalPrefix = "journal/WiredTigerLog."
-
 // Uploads given files to the storage. files may come as 16Mb (by default)
 // blocks in that case it will concat consecutive blocks in one bigger file.
 // For example: f1[0-16], f1[16-24], f1[64-16] becomes f1[0-24], f1[50-16].
-// If this is an incremental, NOT base backup, it will skip unchanged
-// files (Len == 0).
+// If this is an incremental, NOT base backup, it will skip uploading of
+// unchanged files (Len == 0) but add them to the meta as we need know
+// what files shouldn't be restored (those which isn't in the target backup).
 func uploadFiles(ctx context.Context, files []pbm.File, subdir, trimPrefix string, incr bool,
-	stg storage.Storage, comprT compress.CompressionType, comprL *int, l *plog.Event) (journal, data []pbm.File, err error) {
+	stg storage.Storage, comprT compress.CompressionType, comprL *int, l *plog.Event) (data []pbm.File, err error) {
 	if len(files) == 0 {
-		return journal, data, err
+		return data, err
 	}
 
 	trim := func(fname string) string {
-		// path.Clean to get rid of the lading `/` in  case it's
+		// path.Clean to get rid of `/` at the beginning in case it's
 		// left after TrimPrefix. Just for consistent file names in metadata
 		return path.Clean("./" + strings.TrimPrefix(fname, trimPrefix))
 	}
@@ -351,12 +391,21 @@ func uploadFiles(ctx context.Context, files []pbm.File, subdir, trimPrefix strin
 	for _, file := range files[1:] {
 		select {
 		case <-ctx.Done():
-			return nil, nil, ErrCancelled
+			return nil, ErrCancelled
 		default:
 		}
 
-		// skip unchanged files if increment
-		if incr && file.Len == 0 {
+		// Skip uploading unchanged files if incremental
+		// but add them to the meta to keep track of files to be restored
+		// from prev backups. Plus sometimes the cursor can return an offset
+		// beyond the current file size. Such phantom changes shouldn't
+		// be copied. But save meta to have file size.
+		if incr && (file.Len == 0 || file.Off >= file.Size) {
+			file.Off = -1
+			file.Len = -1
+			file.Name = trim(file.Name)
+
+			data = append(data, file)
 			continue
 		}
 
@@ -369,35 +418,28 @@ func uploadFiles(ctx context.Context, files []pbm.File, subdir, trimPrefix strin
 
 		fw, err := writeFile(ctx, wfile, path.Join(subdir, trim(wfile.Name)), stg, comprT, comprL, l)
 		if err != nil {
-			return journal, data, errors.Wrapf(err, "upload file `%s`", wfile.Name)
+			return data, errors.Wrapf(err, "upload file `%s`", wfile.Name)
 		}
 		fw.Name = trim(wfile.Name)
 
-		if strings.HasPrefix(fw.Name, journalPrefix) {
-			journal = append(journal, *fw)
-		} else {
-			data = append(data, *fw)
-		}
+		data = append(data, *fw)
 
 		wfile = file
 	}
 
 	if incr && wfile.Off == 0 && wfile.Len == 0 {
-		return journal, data, nil
+		return data, nil
 	}
 
 	f, err := writeFile(ctx, wfile, path.Join(subdir, trim(wfile.Name)), stg, comprT, comprL, l)
 	if err != nil {
-		return journal, data, errors.Wrapf(err, "upload file `%s`", wfile.Name)
+		return data, errors.Wrapf(err, "upload file `%s`", wfile.Name)
 	}
 	f.Name = trim(wfile.Name)
-	if strings.HasPrefix(f.Name, journalPrefix) {
-		journal = append(journal, *f)
-	} else {
-		data = append(data, *f)
-	}
 
-	return journal, data, nil
+	data = append(data, *f)
+
+	return data, nil
 }
 
 func writeFile(ctx context.Context, src pbm.File, dst string, stg storage.Storage, compression compress.CompressionType, compressLevel *int, l *plog.Event) (*pbm.File, error) {
