@@ -1,6 +1,7 @@
 package pbm
 
 import (
+	"context"
 	"fmt"
 	"os"
 	"reflect"
@@ -40,6 +41,9 @@ func (c Config) String() string {
 	if c.Storage.S3.Credentials.SecretAccessKey != "" {
 		c.Storage.S3.Credentials.SecretAccessKey = "***"
 	}
+	if c.Storage.S3.Credentials.SessionToken != "" {
+		c.Storage.S3.Credentials.SessionToken = "***"
+	}
 	if c.Storage.S3.Credentials.Vault.Secret != "" {
 		c.Storage.S3.Credentials.Vault.Secret = "***"
 	}
@@ -71,14 +75,6 @@ type PITRConf struct {
 	CompressionLevel *int                     `bson:"compressionLevel,omitempty" json:"compressionLevel,omitempty" yaml:"compressionLevel,omitempty"`
 }
 
-func (c *PITRConf) Cast() error {
-	if c.Compression == "" {
-		c.Compression = compress.CompressionTypeS2
-	}
-
-	return nil
-}
-
 // StorageConf is a configuration of the backup storage
 type StorageConf struct {
 	Type       storage.Type `bson:"type" json:"type" yaml:"type"`
@@ -115,7 +111,7 @@ func (s *StorageConf) Path() string {
 			path += "/" + s.S3.Prefix
 		}
 	case storage.Azure:
-		path = fmt.Sprintf(azure.BlobURL, s.Azure.Account, s.Azure.Container)
+		path = fmt.Sprintf(azure.BlobURL, s.Azure.Account) + "/" + s.Azure.Container
 		if s.Azure.Prefix != "" {
 			path += "/" + s.Azure.Prefix
 		}
@@ -130,9 +126,20 @@ func (s *StorageConf) Path() string {
 
 // RestoreConf is config options for the restore
 type RestoreConf struct {
-	BatchSize           int `bson:"batchSize" json:"batchSize,omitempty" yaml:"batchSize,omitempty"` // num of documents to buffer
+	// Logical restore
+	//
+	// num of documents to buffer
+	BatchSize           int `bson:"batchSize" json:"batchSize,omitempty" yaml:"batchSize,omitempty"`
 	NumInsertionWorkers int `bson:"numInsertionWorkers" json:"numInsertionWorkers,omitempty" yaml:"numInsertionWorkers,omitempty"`
 
+	// NumDownloadWorkers sets the num of goroutine would be requesting chunks
+	// during the download. By default, it's set to GOMAXPROCS.
+	NumDownloadWorkers int `bson:"numDownloadWorkers" json:"numDownloadWorkers,omitempty" yaml:"numDownloadWorkers,omitempty"`
+	// MaxDownloadBufferMb sets the max size of the in-memory buffer that is used
+	// to download files from the storage.
+	MaxDownloadBufferMb int `bson:"maxDownloadBufferMb" json:"maxDownloadBufferMb,omitempty" yaml:"maxDownloadBufferMb,omitempty"`
+	DownloadChunkMb     int `bson:"downloadChunkMb" json:"downloadChunkMb,omitempty" yaml:"downloadChunkMb,omitempty"`
+
 	// MongodLocation sets the location of mongod used for internal runs during
 	// physical restore. Will try $PATH/mongod if not set.
 	MongodLocation    string            `bson:"mongodLocation" json:"mongodLocation,omitempty" yaml:"mongodLocation,omitempty"`
@@ -365,6 +372,9 @@ func (p *PBM) GetConfigYaml(fieldRedaction bool) ([]byte, error) {
 		if c.Storage.S3.Credentials.SecretAccessKey != "" {
 			c.Storage.S3.Credentials.SecretAccessKey = "***"
 		}
+		if c.Storage.S3.Credentials.SessionToken != "" {
+			c.Storage.S3.Credentials.SessionToken = "***"
+		}
 		if c.Storage.S3.Credentials.Vault.Secret != "" {
 			c.Storage.S3.Credentials.Vault.Secret = "***"
 		}
@@ -381,19 +391,30 @@ func (p *PBM) GetConfigYaml(fieldRedaction bool) ([]byte, error) {
 }
 
 func (p *PBM) GetConfig() (Config, error) {
+	return getPBMConfig(p.ctx, p.Conn)
+}
+
+func getPBMConfig(ctx context.Context, m *mongo.Client) (Config, error) {
+	res := m.Database(DB).Collection(ConfigCollection).FindOne(ctx, bson.D{})
+	if err := res.Err(); err != nil {
+		return Config{}, errors.WithMessage(err, "get")
+	}
+
 	var c Config
-	res := p.Conn.Database(DB).Collection(ConfigCollection).FindOne(p.ctx, bson.D{})
-	if res.Err() != nil {
-		return Config{}, errors.Wrap(res.Err(), "get")
+	if err := res.Decode(&c); err != nil {
+		return Config{}, errors.WithMessage(err, "decode")
 	}
-	err := res.Decode(&c)
-	if err != nil {
-		return c, errors.Wrap(err, "decode")
+
+	if c.Backup.Compression == "" {
+		c.Backup.Compression = compress.CompressionTypeS2
 	}
-	err = c.PITR.Cast()
-	if err != nil {
-		return c, errors.Wrap(err, "cast options")
+	if c.PITR.Compression == "" {
+		c.PITR.Compression = c.Backup.Compression
 	}
+	if c.PITR.CompressionLevel == nil {
+		c.PITR.CompressionLevel = c.Backup.CompressionLevel
+	}
+
 	return c, nil
 }
 
