@@ -40,16 +40,12 @@ const (
 	LockOpCollection = "pbmLockOp"
 	// BcpCollection is a collection for backups metadata
 	BcpCollection = "pbmBackups"
-	// BcpOldCollection contains a backup of backups metadata
-	BcpOldCollection = "pbmBackups.old"
 	// RestoresCollection is a collection for restores metadata
 	RestoresCollection = "pbmRestores"
 	// CmdStreamCollection is the name of the mongo collection that contains backup/restore commands stream
 	CmdStreamCollection = "pbmCmd"
 	// PITRChunksCollection contains index metadata of PITR chunks
 	PITRChunksCollection = "pbmPITRChunks"
-	// PITRChunksOldCollection contains archived index metadata of PITR chunks
-	PITRChunksOldCollection = "pbmPITRChunks.old"
 	// PBMOpLogCollection contains log of acquired locks (hence run ops)
 	PBMOpLogCollection = "pbmOpLog"
 	// AgentsStatusCollection is an agents registry with its status/health checks
@@ -76,6 +72,7 @@ const (
 	CmdPITRestore   Command = "pitrestore"
 	CmdDeleteBackup Command = "delete"
 	CmdDeletePITR   Command = "deletePitr"
+	CmdCleanup      Command = "cleanup"
 )
 
 func (c Command) String() string {
@@ -98,6 +95,8 @@ func (c Command) String() string {
 		return "Delete"
 	case CmdDeletePITR:
 		return "Delete PITR chunks"
+	case CmdCleanup:
+		return "Cleanup backups and PITR chunks"
 	default:
 		return "Undefined"
 	}
@@ -113,6 +112,7 @@ type Cmd struct {
 	PITRestore *PITRestoreCmd   `bson:"pitrestore,omitempty"`
 	Delete     *DeleteBackupCmd `bson:"delete,omitempty"`
 	DeletePITR *DeletePITRCmd   `bson:"deletePitr,omitempty"`
+	Cleanup    *CleanupCmd      `bson:"cleanup,omitempty"`
 	TS         int64            `bson:"ts"`
 	OPID       OPID             `bson:"-"`
 }
@@ -225,6 +225,10 @@ type DeletePITRCmd struct {
 	OlderThan int64 `bson:"olderthan"`
 }
 
+type CleanupCmd struct {
+	OlderThan primitive.Timestamp `bson:"olderThan"`
+}
+
 func (d DeleteBackupCmd) String() string {
 	return fmt.Sprintf("backup: %s, older than: %d", d.Backup, d.OlderThan)
 }
@@ -279,7 +283,7 @@ func New(ctx context.Context, uri, appName string) (*PBM, error) {
 
 	csvr, err := ConfSvrConn(ctx, client)
 	if err != nil {
-		return nil, errors.Wrap(err, "get config server connetion URI")
+		return nil, errors.Wrap(err, "get config server connection URI")
 	}
 	// no need in this connection anymore, we need a new one with the ConfigServer
 	err = client.Disconnect(ctx)
@@ -482,6 +486,7 @@ type BackupMeta struct {
 	Store            StorageConf              `bson:"store" json:"store"`
 	Size             int64                    `bson:"size" json:"size"`
 	MongoVersion     string                   `bson:"mongodb_version" json:"mongodb_version,omitempty"`
+	FCV              string                   `bson:"fcv" json:"fcv"`
 	StartTS          int64                    `bson:"start_ts" json:"start_ts"`
 	LastTransitionTS int64                    `bson:"last_transition_ts" json:"last_transition_ts"`
 	FirstWriteTS     primitive.Timestamp      `bson:"first_write_ts" json:"first_write_ts"`
@@ -528,7 +533,7 @@ type Condition struct {
 
 type BackupReplset struct {
 	Name             string              `bson:"name" json:"name"`
-	Journal          []File              `bson:"journal,omitempty" json:"journal,omitempty"`
+	Journal          []File              `bson:"journal,omitempty" json:"journal,omitempty"` // not used. left for backward compatibility
 	Files            []File              `bson:"files,omitempty" json:"files,omitempty"`
 	DumpName         string              `bson:"dump_name,omitempty" json:"backup_name,omitempty"`
 	OplogName        string              `bson:"oplog_name,omitempty" json:"oplog_name,omitempty"`
@@ -581,6 +586,9 @@ const (
 	StatusInit  Status = "init"
 	StatusReady Status = "ready"
 
+	// for phys restore, to indicate shards have been stopped
+	StatusDown Status = "down"
+
 	StatusStarting   Status = "starting"
 	StatusRunning    Status = "running"
 	StatusDumpDone   Status = "dumpDone"
@@ -974,6 +982,11 @@ func (p *PBM) GetNodeInfo() (*NodeInfo, error) {
 	return inf, nil
 }
 
+// GetNodeInfo returns mongo node info
+func (p *PBM) GetFeatureCompatibilityVersion() (string, error) {
+	return getFeatureCompatibilityVersion(p.ctx, p.Conn)
+}
+
 // ClusterTime returns mongo's current cluster time
 func (p *PBM) ClusterTime() (primitive.Timestamp, error) {
 	// Make a read to force the cluster timestamp update.
