@@ -41,7 +41,7 @@ func NewSlicer(rs string, cn *pbm.PBM, node *pbm.Node, to storage.Storage, ep pb
 		rs:      rs,
 		span:    int64(pbm.PITRdefaultSpan),
 		storage: to,
-		oplog:   oplog.NewOplogBackup(node),
+		oplog:   oplog.NewOplogBackup(node.Session()),
 		l:       cn.Logger().NewEvent(string(pbm.CmdPITR), "", "", ep.TS()),
 		ep:      ep,
 	}
@@ -212,6 +212,10 @@ func (s *Slicer) copyFromBcp(bcp *pbm.BackupMeta) error {
 	if err != nil {
 		return errors.Wrap(err, "storage copy")
 	}
+	stat, err := s.storage.FileStat(n)
+	if err != nil {
+		return errors.Wrap(err, "file stat")
+	}
 
 	meta := pbm.OplogChunk{
 		RS:          s.rs,
@@ -219,6 +223,7 @@ func (s *Slicer) copyFromBcp(bcp *pbm.BackupMeta) error {
 		Compression: bcp.Compression,
 		StartTS:     bcp.FirstWriteTS,
 		EndTS:       bcp.LastWriteTS,
+		Size:        stat.Size,
 	}
 	err = s.pbm.PITRAddChunk(meta)
 	if err != nil {
@@ -333,7 +338,7 @@ func (s *Slicer) Stream(ctx context.Context, backupSig <-chan *pbm.OPID, compres
 		}
 
 		// in case there is a lock, even a legit one (our own, or backup's one) but it is stale
-		// we should return so the slicer would get thru the lock acquisition again.
+		// we should return so the slicer would get through the lock acquisition again.
 		ts, err := s.pbm.ClusterTime()
 		if err != nil {
 			return errors.Wrap(err, "read cluster time")
@@ -352,7 +357,7 @@ func (s *Slicer) Stream(ctx context.Context, backupSig <-chan *pbm.OPID, compres
 				return errors.Wrap(err, "define last write timestamp")
 			}
 		case pbm.CmdUndefined:
-			return errors.New("undefinded behaviour operation is running")
+			return errors.New("undefined behaviour operation is running")
 		case pbm.CmdBackup:
 			// continue only if we had `backupSig`
 			if !lastSlice || primitive.CompareTimestamp(s.lastTS, sliceTo) == 0 {
@@ -402,7 +407,7 @@ func (s *Slicer) upload(from, to primitive.Timestamp, compression compress.Compr
 	s.oplog.SetTailingSpan(from, to)
 	fname := s.chunkPath(from, to, compression)
 	// if use parent ctx, upload will be canceled on the "done" signal
-	_, err := backup.Upload(context.Background(), s.oplog, s.storage, compression, level, fname, -1)
+	size, err := backup.Upload(context.Background(), s.oplog, s.storage, compression, level, fname, -1)
 	if err != nil {
 		// PITR chunks have no metadata to indicate any failed state and if something went
 		// wrong during the data read we may end up with an already created file. Although
@@ -422,6 +427,7 @@ func (s *Slicer) upload(from, to primitive.Timestamp, compression compress.Compr
 		Compression: compression,
 		StartTS:     from,
 		EndTS:       to,
+		Size:        size,
 	}
 	err = s.pbm.PITRAddChunk(meta)
 	if err != nil {
@@ -471,6 +477,10 @@ func (s *Slicer) backupStartTS(opid string) (ts primitive.Timestamp, err error)
 
 // !!! should be agreed with pbm.PITRmetaFromFName()
 func (s *Slicer) chunkPath(first, last primitive.Timestamp, c compress.CompressionType) string {
+	return ChunkName(s.rs, first, last, c)
+}
+
+func ChunkName(rs string, first, last primitive.Timestamp, c compress.CompressionType) string {
 	ft := time.Unix(int64(first.T), 0).UTC()
 	lt := time.Unix(int64(last.T), 0).UTC()
 
@@ -479,7 +489,7 @@ func (s *Slicer) chunkPath(first, last primitive.Timestamp, c compress.Compressi
 		name.WriteString(pbm.PITRfsPrefix)
 		name.WriteString("/")
 	}
-	name.WriteString(s.rs)
+	name.WriteString(rs)
 	name.WriteString("/")
 	name.WriteString(ft.Format("20060102"))
 	name.WriteString("/")
