@@ -16,6 +16,7 @@ import (
 	"path/filepath"
 	"strconv"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/pkg/errors"
@@ -30,6 +31,8 @@ import (
 	"github.com/percona/percona-backup-mongodb/pbm/compress"
 	"github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
+	"github.com/percona/percona-backup-mongodb/pbm/storage/s3"
+	"github.com/percona/percona-backup-mongodb/version"
 )
 
 const (
@@ -45,8 +48,11 @@ type files struct {
 	BcpName string
 	Cmpr    compress.CompressionType
 	Data    []pbm.File
-	dbpath  string
+
+	// dbpath to cut from destination if there is any (see PBM-1058)
+	dbpath string
 }
+
 type PhysRestore struct {
 	cn     *pbm.PBM
 	node   *pbm.Node
@@ -73,15 +79,16 @@ type PhysRestore struct {
 
 	// path to files on a storage the node will sync its
 	// state with the resto of the cluster
-	syncPathNode    string
-	syncPathRS      string
-	syncPathCluster string
-	syncPathPeers   map[string]struct{}
-	// Shards to participate in restore. Num of shards in bcp could
-	// be less than in the cluster and this is ok.
-	//
+	syncPathNode     string
+	syncPathNodeStat string
+	syncPathRS       string
+	syncPathCluster  string
+	syncPathPeers    map[string]struct{}
+	// Shards to participate in restore.
 	// Only the restore leader would have this info.
 	syncPathShards map[string]struct{}
+	// Non-ConfigServer shards
+	syncPathDataShards map[string]struct{}
 
 	stopHB chan struct{}
 
@@ -204,6 +211,15 @@ func (r *PhysRestore) flush() error {
 	if err != nil {
 		return errors.Wrap(err, "get replset status")
 	}
+
+	if r.nodeInfo.IsConfigSrv() {
+		r.log.Debug("waiting for shards to shutdown")
+		_, err := r.waitFiles(pbm.StatusDown, r.syncPathDataShards, false)
+		if err != nil {
+			return errors.Wrap(err, "wait for datashards to shutdown")
+		}
+	}
+
 	for {
 		inf, err := r.node.GetInfo()
 		if err != nil {
@@ -230,6 +246,14 @@ func (r *PhysRestore) flush() error {
 		return errors.Wrap(err, "shutdown")
 	}
 
+	if r.nodeInfo.IsPrimary {
+		err = r.stg.Save(r.syncPathRS+"."+string(pbm.StatusDown),
+			okStatus(), -1)
+		if err != nil {
+			return errors.Wrap(err, "write replset StatusDown")
+		}
+	}
+
 	r.log.Debug("revome old data")
 	err = removeAll(r.dbpath, r.log)
 	if err != nil {
@@ -517,8 +541,48 @@ const (
 	restoreDone
 )
 
-func (n nodeStatus) is(s nodeStatus) bool {
-	return n&s != 0
+func (n nodeStatus) is(s nodeStatus) bool { return n&s != 0 }
+
+// log buffer that will dump content to the storage on restore
+// finish (whether it's successful or not). It also dumps content
+// and reset buffer when logs size hist the limit.
+type logBuff struct {
+	buf   *bytes.Buffer
+	path  string
+	cnt   int
+	write func(name string, data io.Reader) error
+	limit int64
+	mx    sync.Mutex
+}
+
+func (l *logBuff) Write(p []byte) (n int, err error) {
+	l.mx.Lock()
+	defer l.mx.Unlock()
+	if l.buf.Len()+len(p) > int(l.limit) {
+		err := l.flush()
+		if err != nil {
+			return 0, err
+		}
+	}
+
+	return l.buf.Write(p)
+}
+
+func (l *logBuff) flush() error {
+	fname := fmt.Sprintf("%s.%d.log", l.path, l.cnt)
+	err := l.write(fname, l.buf)
+	if err != nil {
+		return errors.Wrapf(err, "write logs buffer to %s", fname)
+	}
+	l.buf.Reset()
+	l.cnt++
+
+	return nil
+}
+func (l *logBuff) Flush() error {
+	l.mx.Lock()
+	defer l.mx.Unlock()
+	return l.flush()
 }
 
 // Snapshot restores data from the physical snapshot.
@@ -597,6 +661,13 @@ func (r *PhysRestore) Snapshot(cmd *pbm.RestoreCmd, opid pbm.OPID, l *log.Event,
 	l.Debug("%s", pbm.StatusStarting)
 
 	// don't write logs to the mongo anymore
+	// but dump it on storage
+	r.cn.Logger().SefBuffer(&logBuff{
+		buf:   new(bytes.Buffer),
+		path:  fmt.Sprintf("%s/%s/rs.%s/log/%s", pbm.PhysRestoresDir, r.name, r.rsConf.ID, r.nodeInfo.Me),
+		limit: 1 << 20, // 1Mb
+		write: func(name string, data io.Reader) error { return r.stg.Save(name, data, -1) },
+	})
 	r.cn.Logger().PauseMgo()
 
 	_, err = r.toState(pbm.StatusRunning)
@@ -634,10 +705,14 @@ func (r *PhysRestore) Snapshot(cmd *pbm.RestoreCmd, opid pbm.OPID, l *log.Event,
 	progress |= restoreStared
 
 	l.Info("copying backup data")
-	err = r.copyFiles()
+	dstat, err := r.copyFiles()
 	if err != nil {
 		return errors.Wrap(err, "copy files")
 	}
+	err = r.writeStat(dstat)
+	if err != nil {
+		r.log.Warning("write download stat: %v", err)
+	}
 
 	l.Info("preparing data")
 	err = r.prepareData()
@@ -677,6 +752,25 @@ func (r *PhysRestore) Snapshot(cmd *pbm.RestoreCmd, opid pbm.OPID, l *log.Event,
 	return nil
 }
 
+func (r *PhysRestore) writeStat(stat any) error {
+	d := struct {
+		D any `json:"d"`
+	}{
+		D: stat,
+	}
+	b, err := json.Marshal(d)
+	if err != nil {
+		return errors.Wrap(err, "marshal")
+	}
+
+	err = r.stg.Save(r.syncPathNodeStat, bytes.NewBuffer(b), -1)
+	if err != nil {
+		return errors.Wrap(err, "write")
+	}
+
+	return nil
+}
+
 func (r *PhysRestore) dumpMeta(meta *pbm.RestoreMeta, s pbm.Status, msg string) error {
 	name := fmt.Sprintf("%s/%s.json", pbm.PhysRestoresDir, meta.Name)
 	_, err := r.stg.FileStat(name)
@@ -693,7 +787,7 @@ func (r *PhysRestore) dumpMeta(meta *pbm.RestoreMeta, s pbm.Status, msg string)
 	// The meta generated here is more for debugging porpuses (just in case).
 	// `pbm status` and `resync` will always rebuild it from agents' reports
 	// not relying solely on this file.
-	condsm, err := pbm.GetPhysRestoreMeta(meta.Name, r.stg)
+	condsm, err := pbm.GetPhysRestoreMeta(meta.Name, r.stg, r.log)
 	if err == nil {
 		meta.Replsets = condsm.Replsets
 		meta.Status = condsm.Status
@@ -725,7 +819,18 @@ func (r *PhysRestore) dumpMeta(meta *pbm.RestoreMeta, s pbm.Status, msg string)
 	return nil
 }
 
-func (r *PhysRestore) copyFiles() error {
+func (r *PhysRestore) copyFiles() (stat *s3.DownloadStat, err error) {
+	readFn := r.stg.SourceReader
+	if t, ok := r.stg.(*s3.S3); ok {
+		d := t.NewDownload(r.confOpts.NumDownloadWorkers, r.confOpts.MaxDownloadBufferMb, r.confOpts.DownloadChunkMb)
+		readFn = d.SourceReader
+		defer func() {
+			s := d.Stat()
+			stat = &s
+			r.log.Debug("download stat: %s", s)
+		}()
+	}
+	cpbuf := make([]byte, 32*1024)
 	for i := len(r.files) - 1; i >= 0; i-- {
 		set := r.files[i]
 		for _, f := range set.Data {
@@ -733,7 +838,7 @@ func (r *PhysRestore) copyFiles() error {
 			if f.Len != 0 {
 				src += fmt.Sprintf(".%d-%d", f.Off, f.Len)
 			}
-			// cat dbpath from destination is there is any (see PBM-1058)
+			// cut dbpath from destination if there is any (see PBM-1058)
 			fname := f.Name
 			if set.dbpath != "" {
 				fname = strings.TrimPrefix(fname, set.dbpath)
@@ -742,46 +847,51 @@ func (r *PhysRestore) copyFiles() error {
 
 			err := os.MkdirAll(filepath.Dir(dst), os.ModeDir|0o700)
 			if err != nil {
-				return errors.Wrapf(err, "create path %s", filepath.Dir(dst))
+				return stat, errors.Wrapf(err, "create path %s", filepath.Dir(dst))
+			}
+			// if this is a directory, only ensure it is created.
+			if set.BcpName == bcpDir {
+				r.log.Info("create dir <%s>", filepath.Dir(f.Name))
+				continue
 			}
 
 			r.log.Info("copy <%s> to <%s>", src, dst)
-			sr, err := r.stg.SourceReader(src)
+			sr, err := readFn(src)
 			if err != nil {
-				return errors.Wrapf(err, "create source reader for <%s>", src)
+				return stat, errors.Wrapf(err, "create source reader for <%s>", src)
 			}
 			defer sr.Close()
 
 			data, err := compress.Decompress(sr, set.Cmpr)
 			if err != nil {
-				return errors.Wrapf(err, "decompress object %s", src)
+				return stat, errors.Wrapf(err, "decompress object %s", src)
 			}
 			defer data.Close()
 
 			fw, err := os.OpenFile(dst, os.O_WRONLY|os.O_CREATE, f.Fmode)
 			if err != nil {
-				return errors.Wrapf(err, "create/open destination file <%s>", dst)
+				return stat, errors.Wrapf(err, "create/open destination file <%s>", dst)
 			}
 			defer fw.Close()
 			if f.Off != 0 {
 				_, err := fw.Seek(f.Off, io.SeekStart)
 				if err != nil {
-					return errors.Wrapf(err, "set file offset <%s>|%d", dst, f.Off)
+					return stat, errors.Wrapf(err, "set file offset <%s>|%d", dst, f.Off)
 				}
 			}
-			_, err = io.Copy(fw, data)
+			_, err = io.CopyBuffer(fw, data, cpbuf)
 			if err != nil {
-				return errors.Wrapf(err, "copy file <%s>", dst)
+				return stat, errors.Wrapf(err, "copy file <%s>", dst)
 			}
 			if f.Size != 0 {
 				err = fw.Truncate(f.Size)
 				if err != nil {
-					return errors.Wrapf(err, "truncate file <%s>|%d", dst, f.Size)
+					return stat, errors.Wrapf(err, "truncate file <%s>|%d", dst, f.Size)
 				}
 			}
 		}
 	}
-	return nil
+	return stat, nil
 }
 
 func (r *PhysRestore) prepareData() error {
@@ -1090,11 +1200,19 @@ func (r *PhysRestore) startMongo(opts ...string) error {
 const hbFrameSec = 60 * 2
 
 func (r *PhysRestore) init(name string, opid pbm.OPID, l *log.Event) (err error) {
-	r.stg, err = r.cn.GetStorage(l)
+	var cfg pbm.Config
+	cfg, err = r.cn.GetConfig()
+	if err != nil {
+		return errors.Wrap(err, "get pbm config")
+	}
+
+	r.stg, err = pbm.Storage(cfg, l)
 	if err != nil {
 		return errors.Wrap(err, "get storage")
 	}
 
+	r.confOpts = cfg.Restore
+
 	r.mongod = "mongod" // run from $PATH by default
 	if r.confOpts.MongodLocation != "" {
 		r.mongod = r.confOpts.MongodLocation
@@ -1111,11 +1229,24 @@ func (r *PhysRestore) init(name string, opid pbm.OPID, l *log.Event) (err error)
 	r.startTS = time.Now().Unix()
 
 	r.syncPathNode = fmt.Sprintf("%s/%s/rs.%s/node.%s", pbm.PhysRestoresDir, r.name, r.rsConf.ID, r.nodeInfo.Me)
+	r.syncPathNodeStat = fmt.Sprintf("%s/%s/rs.%s/stat.%s", pbm.PhysRestoresDir, r.name, r.rsConf.ID, r.nodeInfo.Me)
 	r.syncPathRS = fmt.Sprintf("%s/%s/rs.%s/rs", pbm.PhysRestoresDir, r.name, r.rsConf.ID)
 	r.syncPathCluster = fmt.Sprintf("%s/%s/cluster", pbm.PhysRestoresDir, r.name)
 	r.syncPathPeers = make(map[string]struct{})
 	for _, m := range r.rsConf.Members {
-		r.syncPathPeers[fmt.Sprintf("%s/%s/rs.%s/node.%s", pbm.PhysRestoresDir, r.name, r.rsConf.ID, m.Host)] = struct{}{}
+		if !m.ArbiterOnly {
+			r.syncPathPeers[fmt.Sprintf("%s/%s/rs.%s/node.%s", pbm.PhysRestoresDir, r.name, r.rsConf.ID, m.Host)] = struct{}{}
+		}
+	}
+	if r.nodeInfo.IsConfigSrv() {
+		sh, err := r.cn.GetShards()
+		if err != nil {
+			return errors.Wrap(err, "get data shards")
+		}
+		r.syncPathDataShards = make(map[string]struct{})
+		for _, s := range sh {
+			r.syncPathDataShards[fmt.Sprintf("%s/%s/rs.%s/rs", pbm.PhysRestoresDir, r.name, s.RS)] = struct{}{}
+		}
 	}
 
 	err = r.hb()
@@ -1251,101 +1382,109 @@ func (r *PhysRestore) setTmpConf() (err error) {
 	return nil
 }
 
+const bcpDir = "__dir__"
+
 // Sets replset files that have to be copied to the target during the restore.
 // For non-incremental backups it's just the content of backups files (data) and
-// journals. For the incrementals it will gather files from preceding backups
-// travelling back in time from the target backup up to the closest base. Journal
-// files would be treated differently. We need to restore only journals from the
-// target (latest) backup. As the data from preceding journals already became
-// a data (checkpoint) in the following backup. But if the target backup
-// contains a chunk of the journal that started in the previous one(s), we should
-// retrieve that beginning.
-// Given `b` is backup, `j` is journals and `b3` is a target:
-// b0[j00], b1[j01, j02], b2[j02.10-16, j03], b3[j03.24-16, j04, j05]
-// journals in needed:
-// b2[j03], b3[j03.24-16, j04, j05]
+// journals. For the incrementals, it will gather files from preceding backups
+// travelling back in time from the target backup up to the closest base.
+// `Off == -1 && Len == -1` means the file remains unchanged since the last
+// backup and there is no data in this backup. We need such info in
+// the target backup to know which files from preceding backups should be
+// restored. Only files listed in the target backup will be restored.
 //
 // The restore should be done in reverse order. Applying files (diffs)
 // starting from the base and moving forward in time up to the target backup.
 func (r *PhysRestore) setBcpFiles() (err error) {
 	bcp := r.bcp
-	partj := ""
 
 	rs := getRS(bcp, r.nodeInfo.SetName)
-	var prefix string
-	if is, prx := checkDBpath(rs.Files); is {
-		r.log.Debug("detected PBM-1058 issue in backup %s, dbpath is %s", bcp.Name, prx)
-		prefix = prx
+	if rs == nil {
+		return errors.Errorf("no data in the backup for the replica set %s", r.nodeInfo.SetName)
 	}
-	if is, prx := checkDBpath(rs.Journal); is {
-		r.log.Debug("detected PBM-1058 issue in backup %s, dbpath is %s", bcp.Name, prx)
-		if prefix == "" {
-			prefix = prx
-		}
-	}
-	data := files{
-		BcpName: bcp.Name,
-		Cmpr:    bcp.Compression,
-		Data:    append([]pbm.File{}, rs.Files...),
-		dbpath:  prefix,
-	}
-	for _, j := range rs.Journal {
-		data.Data = append(data.Data, j)
-		if j.Len != 0 && j.Len != j.Size {
-			partj = j.Name
-		}
-	}
-	r.files = append(r.files, data)
 
-	for bcp.SrcBackup != "" {
+	targetFiles := make(map[string]bool)
+	for _, f := range append(rs.Files, rs.Journal...) {
+		targetFiles[f.Name] = false
+	}
+
+	for {
+		data := files{
+			BcpName: bcp.Name,
+			Cmpr:    bcp.Compression,
+			Data:    []pbm.File{},
+		}
+		// PBM-1058
+		var is1058 bool
+		for _, f := range append(rs.Files, rs.Journal...) {
+			if _, ok := targetFiles[f.Name]; ok && f.Off >= 0 && f.Len >= 0 {
+				data.Data = append(data.Data, f)
+				targetFiles[f.Name] = true
+
+				if data.dbpath == "" {
+					is1058, data.dbpath = findDBpath(f.Name)
+				}
+			}
+		}
+		if is1058 {
+			r.log.Debug("issue PBM-1058 detected in backup %s, dbpath is %s", bcp.Name, data.dbpath)
+		}
+
+		r.files = append(r.files, data)
+
+		if bcp.SrcBackup == "" {
+			break
+		}
+
 		r.log.Debug("get src %s", bcp.SrcBackup)
 		bcp, err = r.cn.GetBackupMeta(bcp.SrcBackup)
 		if err != nil {
 			return errors.Wrapf(err, "get source backup")
 		}
-
 		rs = getRS(bcp, r.nodeInfo.SetName)
-		data := append([]pbm.File{}, rs.Files...)
+	}
 
-		if partj != "" {
-			for _, j := range rs.Journal {
-				if partj == j.Name {
-					data = append(data, j)
-				}
-				if j.Len == 0 || j.Len == j.Size {
-					partj = ""
-				}
+	// Directories only. Incremental $backupCusor returns collections that
+	// were created but haven't ended up in the checkpoint yet in the format
+	// if they don't belong to this backup (see PBM-1063). PBM doesn't copy
+	// such files. But they are already in WT metadata. PSMDB (WT) handles
+	// this by creating such files during the start. But fails to do so if
+	// it runs with `directoryPerDB` option. Namely fails to create a directory
+	// for the collections. So we have to detect and create these directories
+	// during the restore.
+	var dirs []pbm.File
+	dirsm := make(map[string]struct{})
+	for f, was := range targetFiles {
+		if !was {
+			dir := path.Dir(f)
+			if _, ok := dirsm[dir]; dir != "." && !ok {
+				dirs = append(dirs, pbm.File{
+					Name: f,
+					Off:  -1,
+					Len:  -1,
+					Size: -1,
+				})
+				dirsm[dir] = struct{}{}
 			}
 		}
-		var prefix string
-		if is, prx := checkDBpath(rs.Files); is {
-			r.log.Debug("detected PBM-1058 issue in backup %s, dbpath is %s", bcp.Name, prx)
-			prefix = prx
-		}
+	}
+
+	if len(dirs) > 0 {
 		r.files = append(r.files, files{
-			BcpName: bcp.Name,
-			Cmpr:    bcp.Compression,
-			Data:    data,
-			dbpath:  prefix,
+			BcpName: bcpDir,
+			Data:    dirs,
 		})
 	}
+
 	return nil
 }
 
 // Checks if dbpath exists in the file name (affected by PBM-1058) and
 // returns it.
 // We suppose that "journal" will always be present in the backup and it is
-// always in the dbpath root and doesn't contain subdirs, only files.
-func checkDBpath(files []pbm.File) (exists bool, prefix string) {
-	for _, f := range files {
-		exists, prefix = findDBpath(f.Name)
-		if exists && prefix != "" {
-			return exists, prefix
-		}
-	}
-
-	return exists, prefix
-}
+// always in the dbpath root and doesn't contain subdirs, only files. Having
+// a leading `/` indicates that restore was affected by PBM-1058 but only
+// with journal files we may detect the exact prefix.
 func findDBpath(fname string) (is bool, prefix string) {
 	if !strings.HasPrefix(fname, "/") {
 		return false, ""
@@ -1394,6 +1533,10 @@ func (r *PhysRestore) prepareBackup(backupName string) (err error) {
 		return errors.Errorf("backup wasn't successful: status: %s, error: %s", r.bcp.Status, r.bcp.Error())
 	}
 
+	if !version.CompatibleWith(r.bcp.PBMVersion, pbm.BreakingChangesMap[r.bcp.Type]) {
+		return errors.Errorf("backup version (v%s) is not compatible with PBM v%s", r.bcp.PBMVersion, version.DefaultInfo.Version)
+	}
+
 	mgoV, err := r.node.GetMongoVersion()
 	if err != nil || len(mgoV.Version) < 1 {
 		return errors.Wrap(err, "define mongo version")
@@ -1420,20 +1563,17 @@ func (r *PhysRestore) prepareBackup(backupName string) (err error) {
 	}
 
 	fl := make(map[string]pbm.Shard, len(s))
+	r.syncPathShards = make(map[string]struct{})
 	for _, rs := range s {
 		fl[rs.RS] = rs
+		r.syncPathShards[fmt.Sprintf("%s/%s/rs.%s/rs", pbm.PhysRestoresDir, r.name, rs.RS)] = struct{}{}
 	}
 
-	r.syncPathShards = make(map[string]struct{})
 	var nors []string
 	for _, sh := range r.bcp.Replsets {
-		rs, ok := fl[sh.Name]
-		if !ok {
+		if _, ok := fl[sh.Name]; !ok {
 			nors = append(nors, sh.Name)
-			continue
 		}
-
-		r.syncPathShards[fmt.Sprintf("%s/%s/rs.%s/rs", pbm.PhysRestoresDir, r.name, rs.RS)] = struct{}{}
 	}
 
 	if len(nors) > 0 {
