@@ -18,6 +18,7 @@ import (
 	"github.com/percona/percona-backup-mongodb/pbm/archive"
 	"github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
+	"github.com/percona/percona-backup-mongodb/pbm/storage/s3"
 	"github.com/percona/percona-backup-mongodb/version"
 )
 
@@ -48,7 +49,7 @@ func (p *PBM) ResyncStorage(l *log.Event) error {
 	l.Debug("got physical restores list: %v", len(rstrs))
 	for _, rs := range rstrs {
 		rname := strings.TrimSuffix(rs.Name, ".json")
-		rmeta, err := GetPhysRestoreMeta(rname, stg)
+		rmeta, err := GetPhysRestoreMeta(rname, stg, l)
 		if err != nil {
 			l.Error("get meta for restore %s: %v", rs.Name, err)
 			if rmeta == nil {
@@ -73,13 +74,14 @@ func (p *PBM) ResyncStorage(l *log.Event) error {
 	}
 	l.Debug("got backups list: %v", len(bcps))
 
-	err = p.moveCollection(BcpCollection, BcpOldCollection)
+	_, err = p.Conn.Database(DB).Collection(BcpCollection).DeleteMany(p.ctx, bson.M{})
 	if err != nil {
-		return errors.Wrapf(err, "copy current backups meta from %s to %s", BcpCollection, BcpOldCollection)
+		return errors.Wrapf(err, "clean up %s", BcpCollection)
 	}
-	err = p.moveCollection(PITRChunksCollection, PITRChunksOldCollection)
+
+	_, err = p.Conn.Database(DB).Collection(PITRChunksCollection).DeleteMany(p.ctx, bson.M{})
 	if err != nil {
-		return errors.Wrapf(err, "copy current pitr meta from %s to %s", PITRChunksCollection, PITRChunksOldCollection)
+		return errors.Wrapf(err, "clean up %s", PITRChunksCollection)
 	}
 
 	var ins []interface{}
@@ -123,13 +125,14 @@ func (p *PBM) ResyncStorage(l *log.Event) error {
 
 	var pitr []interface{}
 	for _, f := range pitrf {
-		_, err := stg.FileStat(PITRfsPrefix + "/" + f.Name)
+		stat, err := stg.FileStat(PITRfsPrefix + "/" + f.Name)
 		if err != nil {
 			l.Warning("skip pitr chunk %s/%s because of %v", PITRfsPrefix, f.Name, err)
 			continue
 		}
 		chnk := PITRmetaFromFName(f.Name)
 		if chnk != nil {
+			chnk.Size = stat.Size
 			pitr = append(pitr, chnk)
 		}
 	}
@@ -211,32 +214,7 @@ func checkFile(stg storage.Storage, filename string) error {
 	return nil
 }
 
-func (p *PBM) moveCollection(coll, as string) error {
-	err := p.Conn.Database(DB).Collection(as).Drop(p.ctx)
-	if err != nil {
-		return errors.Wrap(err, "failed to remove old archive from backups metadata")
-	}
-
-	cur, err := p.Conn.Database(DB).Collection(coll).Find(p.ctx, bson.M{})
-	if err != nil {
-		return errors.Wrap(err, "get current data")
-	}
-	for cur.Next(p.ctx) {
-		_, err = p.Conn.Database(DB).Collection(as).InsertOne(p.ctx, cur.Current)
-		if err != nil {
-			return errors.Wrapf(err, "insert")
-		}
-	}
-
-	if cur.Err() != nil {
-		return cur.Err()
-	}
-
-	_, err = p.Conn.Database(DB).Collection(coll).DeleteMany(p.ctx, bson.M{})
-	return errors.Wrap(err, "remove current data")
-}
-
-func GetPhysRestoreMeta(restore string, stg storage.Storage) (rmeta *RestoreMeta, err error) {
+func GetPhysRestoreMeta(restore string, stg storage.Storage, l *log.Event) (rmeta *RestoreMeta, err error) {
 	mjson := filepath.Join(PhysRestoresDir, restore) + ".json"
 	_, err = stg.FileStat(mjson)
 	if err != nil && !errors.Is(err, storage.ErrNotExist) {
@@ -255,7 +233,7 @@ func GetPhysRestoreMeta(restore string, stg storage.Storage) (rmeta *RestoreMeta
 		}
 	}
 
-	condsm, err := ParsePhysRestoreStatus(restore, stg)
+	condsm, err := ParsePhysRestoreStatus(restore, stg, l)
 	if err != nil {
 		return rmeta, errors.Wrap(err, "parse physical restore status")
 	}
@@ -275,6 +253,7 @@ func GetPhysRestoreMeta(restore string, stg storage.Storage) (rmeta *RestoreMeta
 	rmeta.Hb = condsm.Hb
 	rmeta.Conditions = condsm.Conditions
 	rmeta.Type = PhysicalBackup
+	rmeta.Stat = condsm.Stat
 
 	return rmeta, err
 }
@@ -282,7 +261,7 @@ func GetPhysRestoreMeta(restore string, stg storage.Storage) (rmeta *RestoreMeta
 // ParsePhysRestoreStatus parses phys restore's sync files and creates RestoreMeta.
 //
 // On files format, see comments for *PhysRestore.toState() in pbm/restore/physical.go
-func ParsePhysRestoreStatus(restore string, stg storage.Storage) (*RestoreMeta, error) {
+func ParsePhysRestoreStatus(restore string, stg storage.Storage, l *log.Event) (*RestoreMeta, error) {
 	rfiles, err := stg.List(PhysRestoresDir+"/"+restore, "")
 	if err != nil {
 		return nil, errors.Wrap(err, "get files")
@@ -359,6 +338,28 @@ func ParsePhysRestoreStatus(restore string, stg storage.Storage) (*RestoreMeta,
 					rs.rs.LastTransitionTS = l.Timestamp
 					rs.rs.Error = l.Error
 				}
+			case "stat":
+				src, err := stg.SourceReader(filepath.Join(PhysRestoresDir, restore, f.Name))
+				if err != nil {
+					l.Error("get stat file %s: %v", f.Name, err)
+					break
+				}
+				if meta.Stat == nil {
+					meta.Stat = &RestoreStat{Download: make(map[string]map[string]s3.DownloadStat)}
+				}
+				st := struct {
+					D s3.DownloadStat `json:"d"`
+				}{}
+				err = json.NewDecoder(src).Decode(&st)
+				if err != nil {
+					l.Error("unmarshal stat file %s: %v", f.Name, err)
+					break
+				}
+				if _, ok := meta.Stat.Download[rsName]; !ok {
+					meta.Stat.Download[rsName] = make(map[string]s3.DownloadStat)
+				}
+				nName := strings.Join(p[1:], ".")
+				meta.Stat.Download[rsName][nName] = st.D
 			}
 			rss[rsName] = rs
 
@@ -380,9 +381,29 @@ func ParsePhysRestoreStatus(restore string, stg storage.Storage) (*RestoreMeta,
 		}
 	}
 
+	// If all nodes in the rs are in "error" state, set rs as "error".
+	// We have "partlyDone", so it's not an error if at least one node is "done".
 	for _, rs := range rss {
+		noerr := 0
+		nodeErr := ""
 		for _, node := range rs.nodes {
 			rs.rs.Nodes = append(rs.rs.Nodes, node)
+			if node.Status != StatusError {
+				noerr++
+			}
+			if node.Error != "" {
+				nodeErr = node.Error
+			}
+		}
+		if noerr == 0 {
+			rs.rs.Status = StatusError
+			if rs.rs.Error == "" {
+				rs.rs.Error = nodeErr
+			}
+			meta.Status = StatusError
+			if meta.Error == "" {
+				meta.Error = nodeErr
+			}
 		}
 		meta.Replsets = append(meta.Replsets, rs.rs)
 	}
